/***************************************************************************
 *   Copyright (C) 2008 by Pino Toscano <pino@kde.org>                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef _OKULAR_UTILS_P_H_
#define _OKULAR_UTILS_P_H_

class QIODevice;

namespace Okular
{
void copyQIODevice(QIODevice *from, QIODevice *to);

/**
 * Return a rotation matrix corresponding to the @p rotation enumeration.
 */
QTransform buildRotationMatrix(Rotation rotation);

}

#endif
